require 'spec_helper'

describe Quby::Compiler::Outputs::RoquaSerializer do
  it 'generates outcome tables schema from questionnaire defined outcome tables' do
    questionnaire = dsl("test") do
      title "Test Quest"

      score_schema :score1, 'Score 1' do
        subscore(:value, 'Waarde', export_key: :score1) { 1 }
        subscore(:int, 'Interpretatie', export_key: :score1_i) { 'Wah' }
      end

      score_schema :score0, 'Score 0' do
        subscore(:value, 'Nog iets', export_key: :score0) { 'Iets' }
      end

      outcome_table name: 'Alleen score 0',
                    key: :only_score0,
                    score_keys: [:score0],
                    default_collapsed: true,
                    subscore_keys: %i[value]

      outcome_table name: 'Scores',
                    key: :scores,
                    score_keys: [:score0, :score1],
                    subscore_keys: %i[value int]
    end
    serializer = described_class.new(questionnaire)

    expected = {
      outcome_tables_schema:
        {headers: {int: "Interpretatie", value: "Nog iets"},
         tables: {only_score0: {default_collapsed: true,
                                name: "Alleen score 0",
                                score_keys: [:score0],
                                subscore_keys: [:value]},
                  scores: {name: "Scores",
                           score_keys: [:score0, :score1],
                           subscore_keys: [:value, :int]}}}
    }
    expect(serializer.as_json).to include(expected)
  end

  it 'generates outcome tables schema from score schema outcome tables' do
    questionnaire = dsl("test") do
      title "Test Quest"

      score_schema :score0, 'Score 0' do
        # By default, scores end up in the :main table
        subscore :something, 'Nog iets', export_key: :score0 do
          'Iets'
        end
      end

      score_schema :score1, 'Score 1' do
        subscore :value, 'Waarde', export_key: :score1, outcome_table: :values do
          1
        end
        subscore :int, 'Interpretatie', export_key: :score1_i, outcome_table: :values do
          'Wah'
        end
      end

      score_schema :score2, 'Score 2' do
        subscore :value, 'Waarde', export_key: :score2, outcome_table: :values do
          2
        end
      end

      score_schema :score3, 'Score 3' do
        subscore :mean, 'Gemiddelde', export_key: :score3, outcome_table: :means do
          1
        end
      end

      score_schema :score4, 'Score 4' do
        subscore :value, 'Waarde', export_key: :score4, outcome_table: nil do
          2
        end
        subscore :mean, 'Gemiddelde', export_key: :score4_m, outcome_table: :means do
          1
        end
      end
    end

    serializer = described_class.new(questionnaire)

    expect(serializer.as_json).to \
      include({
                outcome_tables_schema: {
                  headers: {int: "Interpretatie", mean: "Gemiddelde", something: "Nog iets", value: "Waarde"},
                  tables: {
                    main: {score_keys: [:score0], subscore_keys: [:something]},
                    values: {score_keys: [:score1, :score2], subscore_keys: [:value, :int]},
                    means: {score_keys: [:score3, :score4], subscore_keys: [:mean]}
                  }
                }
              })
  end

  def dsl(key, &block)
    Quby::Compiler::DSL.build(key, nil, &block)
  end
end